@@ -255,17 +255,25 @@ PHP_FUNCTION(gearman_client_set_timeout) {
 }
 /* }}} */
 
-/* {{{ proto bool gearman_client_add_server(object client [, string host [, int port]])
+/* {{{ proto bool gearman_client_add_server(object client [, string host [, int
+ * port [, bool setupExceptionHandler ]]])
    Add a job server to a client. This goes into a list of servers than can be used to run tasks. No socket I/O happens here, it is just added to a list. */
 PHP_FUNCTION(gearman_client_add_server) {
         char *host = NULL;
         size_t host_len = 0;
         zend_long port = 0;
+        zend_bool setupExceptionHandler = 1;
 
         gearman_client_obj *obj;
         zval *zobj;
 
-        if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O|sl", &zobj, gearman_client_ce, &host, &host_len, &port) == FAILURE) {
+        if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O|slb",
+					&zobj,
+					gearman_client_ce,
+					&host,
+					&host_len,
+					&port,
+					&setupExceptionHandler) == FAILURE) {
                 RETURN_FALSE;
         }            
         obj = Z_GEARMAN_CLIENT_P(zobj);
@@ -275,26 +283,32 @@ PHP_FUNCTION(gearman_client_add_server) {
                 php_error_docref(NULL, E_WARNING, "%s",
                                                  gearman_client_error(&(obj->client)));
                 RETURN_FALSE;                         
-        }            
+        }
 
-        if (!gearman_client_set_server_option(&(obj->client), "exceptions", (sizeof("exceptions") - 1))) {
+        if (setupExceptionHandler && !gearman_client_set_server_option(&(obj->client), "exceptions", (sizeof("exceptions") - 1))) {
                 GEARMAN_EXCEPTION("Failed to set exception option", 0);
-        }            
+        }
 
         RETURN_TRUE;
 }
 /* }}} */
 
-/* {{{ proto bool gearman_client_add_servers(object client [, string servers])
+/* {{{ proto bool gearman_client_add_servers(object client [, string servers [, bool setupExceptionHandler ]])
    Add a list of job servers to a client. This goes into a list of servers that can be used to run tasks. No socket I/O happens here, it is just added to a list. */
 PHP_FUNCTION(gearman_client_add_servers) {
         char *servers = NULL;
         size_t servers_len = 0;
+        zend_bool setupExceptionHandler = 1;
 
         gearman_client_obj *obj;
         zval *zobj;
 
-        if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O|s", &zobj, gearman_client_ce, &servers, &servers_len) == FAILURE) {
+        if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O|sb",
+							&zobj,
+							gearman_client_ce,
+							&servers,
+							&servers_len,
+							&setupExceptionHandler) == FAILURE) {
                 RETURN_FALSE;
         }
         obj = Z_GEARMAN_CLIENT_P(zobj);
@@ -306,7 +320,7 @@ PHP_FUNCTION(gearman_client_add_servers) {
                 RETURN_FALSE;
         }
 
-        if (!gearman_client_set_server_option(&(obj->client), "exceptions", (sizeof("exceptions") - 1))) {
+        if (setupExceptionHandler && !gearman_client_set_server_option(&(obj->client), "exceptions", (sizeof("exceptions") - 1))) {
                 GEARMAN_EXCEPTION("Failed to set exception option", 0);
         }
 
@@ -437,7 +451,7 @@ static void gearman_client_do_background_work_handler(gearman_return_t (*do_back
         gearman_client_obj *obj;
         zval *zobj;
 
-        if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Oss|s", &zobj, gearman_client_ce,
+        if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "Oss|s!", &zobj, gearman_client_ce,
                                                         &function_name, &function_name_len,
                                                         &workload, &workload_len,
                                                         &unique, &unique_len) == FAILURE) {
@@ -1085,6 +1099,10 @@ PHP_FUNCTION(gearman_client_set_exception_callback) {
 	}
 	obj = Z_GEARMAN_CLIENT_P(zobj);
 
+        if (!gearman_client_set_server_option(&(obj->client), "exceptions", (sizeof("exceptions") - 1))) {
+                GEARMAN_EXCEPTION("Failed to set exception option", 0);
+        }
+
 	/* check that the function is callable */
 	if (! zend_is_callable(zexception_fn, 0, &callable)) {
 		php_error_docref(NULL, E_WARNING, "function %s is not callable", callable->val);
@@ -1224,3 +1242,23 @@ PHP_FUNCTION(gearman_client_set_context) {
         RETURN_TRUE;
 }
 /* }}} */
+
+/* {{{ proto bool GearmanClient::enableExceptionHandler()
+   Enable exception handling to be used by exception callback function
+   GearmanClient::setExceptionCallback */
+PHP_FUNCTION(gearman_client_enable_exception_handler) {
+        gearman_client_obj *obj;
+        zval *zobj;
+
+        if (zend_parse_method_parameters(ZEND_NUM_ARGS(), getThis(), "O", &zobj, gearman_client_ce) == FAILURE) {
+                RETURN_FALSE;
+        }
+        obj = Z_GEARMAN_CLIENT_P(zobj);
+
+        if (!gearman_client_set_server_option(&(obj->client), "exceptions", (sizeof("exceptions") - 1))) {
+                GEARMAN_EXCEPTION("Failed to set exception option", 0);
+        }
+
+        RETURN_TRUE;
+}
+/* }}} */
